import numpy as np
from data_utils import process_data, fnn_processing, f1, scorer
from keras import optimizers
from keras.models import Sequential
from keras.layers import InputLayer, Dense

"""
For this task will we train a fully connected neural network, or deep neural network
to classify locations in text. We will use the Keras library to make things easier for us.

Some of the helper code can be found in data_utils.py if you're interested but you
don't need to modify anything in that file to solve the task.

Below you will find three functions, train and eval, which are called further down in the code
and build, which is used to construct a Neural Network model based on the Keras Sequential API:
https://keras.io/models/sequential/

Your task is to build a Deep Neural Network operating on tfidf vector data which
solves the classification task. Since we are not interested in specific feature engineering
the only thing we will adjust are the network architecture and hyperparameters.

All the data is already loaded and processed into Train and Test Sets for you.
Feel free to inspect the data if you want to learn how it is structured.

Familiarize yourself with the relevant parts of the Keras API.
The following links might be helpful:
https://keras.io/layers/core
https://keras.io/losses/
https://keras.io/optimizers/
https://keras.io/metrics/

Relevant Hyperparameters include:
Number of Training Epochs
Batch Size
Loss Function
Optimizer and Learning Rate
Evaluation Metrics

There might be more things that are worth adjusting.
"""

def train(model, batch_size=32, num_epochs=5):
    model.fit(train_X, train_y, batch_size=batch_size, epochs=num_epochs, validation_split=0.2)
    return True


def eval(model):
    print("Predicting Test Output")
    output = model.predict(test_X, verbose=1)
    predict_labels = np.argmax(output, axis=-1)
    gold_labels = np.argmax(test_y, axis=-1)
    scorer(gold_labels, predict_labels)

"""
The function process_data has a parameter called slice_size which determines how
much of the available data we will load to do our training. If you remove this
parameter the full dataset will be used, but beware that this will lead to longer
training time.
"""
train_docs, test_docs, train_labels, test_labels, word2index, _ = process_data('full_data.txt', slice_size=-1)
train_X, test_X, train_y, test_y = fnn_processing(train_docs, test_docs,
                                                  train_labels, test_labels, len(word2index))
_, n_words = train_X.shape


"""
Adjust the model to what you think would serve the task better. The only givens
are the input layer, with the size corresponding to our tfidf vector length and
an output layer of 3, corresponding to our 3 IOB tags.
"""
def build_model(input_size):
    model = Sequential()
    ######### EXPAND THE MODEL #########
    model.add(InputLayer(input_shape=(input_size,)))
    model.add(Dense(3))

    sgd = optimizers.SGD(lr=50)
    model.compile(loss='categorical_crossentropy',
                  optimizer=sgd,
                  metrics=['acc'])
    return model


num_epochs = 2
batch_size = 8

model = build_model(n_words)
model.summary()

"""
When you are ready for training + evaluation uncomment the two lines below.
"""
#train(model, batch_size=batch_size, num_epochs=num_epochs)
#eval(model)
######### UNCOMMENT ABOVE TO BEGIN TRAINING #########
